/*
 *  stereo -- Stereo matching software
 *  $Id: adjust.c 197 2004-07-20 16:07:20Z eric $
 *  Copyright (c) 2004 Sam Hocevar <sam@zoy.org>
 *                     Éric Beets <ericbeets@free.fr>
 *  All Rights Reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA
 */

#include <stdint.h>
#include <stdio.h>

#include "defs.h"
#include "main.h"
#include "geometry.h"

static pt2d_t transform(projection_t *data,
                        pt2d_t from);

/* Adjust relative height of destination images */
int adjust(pair_t *pair)
{
    double min = 999999, height;
    int closer = 0, i;
    pt2d_t centre, from, to;

    centre.x = pair->image[LEFT].width / 2;
    centre.y = pair->image[LEFT].height / 2;

    for(i = 1; i < pair->projection[LEFT].scanline_count; i++)
    {
        pt2d_t a = pair->projection[LEFT].scanline[i].a;
        pt2d_t b = pair->projection[LEFT].scanline[i].b;

        double dy = (centre.x - a.x) / (b.x - a.x) * (b.y - a.y);

        if(abs(dy + a.y - centre.y) < min)
        {
            min = abs(dy + a.y - centre.y);
            closer = i;
        }
    }

    printf("scanline trouve: %d/%d\n", closer, pair->projection[LEFT].scanline_count);

    from.x = (pair->projection[LEFT].scanline[closer].a.x
               + pair->projection[LEFT].scanline[closer].b.x) / 2;
    from.y = (pair->projection[LEFT].scanline[closer].a.y
               + pair->projection[LEFT].scanline[closer].b.y) / 2;
    to = transform(&pair->projection[LEFT], from);
    height = to.y;

    from.x = (pair->projection[RIGHT].scanline[closer].a.x
               + pair->projection[RIGHT].scanline[closer].b.x) / 2;
    from.y = (pair->projection[RIGHT].scanline[closer].a.y
               + pair->projection[RIGHT].scanline[closer].b.y) / 2;
    to = transform(&pair->projection[RIGHT], from);

    pair->adjust = height - to.y;

    printf("ajustement %g\n", pair->adjust);

    return 0;
}

static pt2d_t transform(projection_t *data,
                        pt2d_t from)
{
    pt3d_t pt3d;
    pt2d_t to;

    pt3d.x = from.x;
    pt3d.y = from.y;
    pt3d.z = data->d;

    pt3d = multiply(1 / scalar_product(pt3d, data->Os), pt3d);
    pt3d = multiply(scalar_product(data->Os, data->Os), pt3d);

    to.x = scalar_product(pt3d, data->vx);
    to.y = scalar_product(pt3d, data->vy);

    return to;
}

