/*
 *  stereo -- Stereo matching software
 *  $Id: main.c 196 2004-07-20 12:59:27Z sam $
 *  Copyright (c) 2004 Sam Hocevar <sam@zoy.org>
 *                     Éric Beets <ericbeets@free.fr>
 *  All Rights Reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA
 */

#include <stdint.h>
#include <stdio.h>
#include <getopt.h>

//// REMOVE THAT CODE
#include <cv.h>
#include <highgui.h>
//// ----------------

#include "defs.h"
#include "main.h"
#include "image.h"

static int do_stereo(pair_t *pair);
static int undo_stereo(pair_t *pair);

int Antialiasing = 1;
int Normalise = 1;

int main(int argc, char** argv)
{
    char *left_name, *right_name;
    pair_t pair;
    double factor=1;

    while(1)
    {
        int c = getopt(argc, argv, "ht:f:n");
        if(c == -1)
            break;

        switch(c)
        {
            case 'h':
                printf("usage: %s [left_image right_image] [-t <tracker>] [-n]\n", argv[0]);
                return 0;
            case 't':
                switch(atoi(optarg))
                {
                    case 1: track_mode = TRACKV1; break;
                    case 2: track_mode = TRACKV2; break;
                    case 3: track_mode = TRACKV3; break;
                    default: track_mode = TRACKV4; break;
                }
                break;
            case 'f':
                factor = atof(optarg);
                break;
            case 'n':
                Normalise = 0;
                break;
            default:
                printf("%s: bad opt `%s'\n", argv[0], argv[optind]);
                return -1;
        }
    }

    left_name = "test.bmp";
    left_name = "../images/TruckL.bmp";
    right_name = "../images/TruckR.bmp";

    if(optind < argc)
    {
        if(argc - optind == 2)
        {
            left_name = argv[optind];
            right_name = argv[optind + 1];
        }
        else
        {
            fprintf(stderr, "%s: wrong argument count\n", argv[0]);
            return -1;
        }
    }

//// REMOVE THAT CODE
    cvInitSystem(argc, argv);
//// ----------------

    if(image_load(&pair.image[LEFT], left_name))
        return -1;
    if(image_load(&pair.image[RIGHT], right_name))
        return -1;

    Resize(&pair.image[LEFT],factor);
    Resize(&pair.image[RIGHT],factor);
	

	{
		image_t *tmp,*tmp1,*tmp2;
		tmp = malloc(sizeof(image_t));
		tmp1 = malloc(sizeof(image_t));
		tmp2 = malloc(sizeof(image_t));
		//acp(&pair.image[LEFT],tmp);
		convert_BGR2GRAY(&pair.image[LEFT],tmp);
		image_new(tmp1,tmp->width,tmp->height,1);
		image_new(tmp2,tmp->width,tmp->height,1);
		cvCvtColor((&(pair.image[LEFT]))->_private,tmp1->_private,CV_BGR2GRAY);
		ng(&pair.image[LEFT],tmp2);
		image_display(tmp2);		
		image_display(tmp1);		
		image_display(tmp);	
	}
    
	image_display(&pair.image[LEFT]);	
	image_display(&pair.image[RIGHT]);	
    
    
    do_stereo(&pair);

/*    initGL(argc,argv);

    OpenGL_Carte3d(    &pair.image[LEFT],
                    &pair.depthmap[LEFT]);

    loopGL();
*/
//// REMOVE THAT CODE
    while((unsigned char)cvWaitKey(0) != 0x1b)
        ;
    
    cvDestroyAllWindows();
//// ----------------

    undo_stereo(&pair);

    return 0;
}

static int do_stereo(pair_t *pair)
{
    if(Normalise)
    {
        if(singularities(&pair->image[LEFT],
                         &pair->projection[LEFT],
                         &pair->image[RIGHT],
                         &pair->projection[RIGHT]))
            return -1;

        if(scanlines(&pair->image[LEFT],
                     &pair->projection[LEFT],
                     &pair->image[RIGHT],
                     &pair->projection[RIGHT]))
            return -1;

//        image_display(&pair->image[LEFT]);
//        image_display(&pair->image[RIGHT]);

        if(parameters(&pair->image[LEFT],
                      &pair->projection[LEFT]))
            return -1;
        if(limits(&pair->image[LEFT],
                  &pair->projection[LEFT]))
            return -1;
        if(normalise(&pair->image[LEFT],
                     &pair->projection[LEFT],
                     &pair->image_norm[LEFT]))
            return -1;

        image_display(&pair->image_norm[LEFT]);

        if(parameters(&pair->image[RIGHT],
                      &pair->projection[RIGHT]))
            return -1;
        if(limits(&pair->image[RIGHT],
                  &pair->projection[RIGHT]))
            return -1;
        if(normalise(&pair->image[RIGHT],
                     &pair->projection[RIGHT],
                     &pair->image_norm[RIGHT]))
            return -1;

        image_display(&pair->image_norm[RIGHT]);

        if(adjust(pair))
            return -1;
    }
    else
    {
        pair->image_norm[LEFT] = pair->image[LEFT];
        pair->image_norm[RIGHT] = pair->image[RIGHT];
        pair->adjust = 0;
    }

    if(matching(pair))
        return -1;

    image_display(&pair->depthmap_norm[LEFT]);
//    image_display(&pair->depthmap_norm[RIGHT]);

    if(Normalise)
    {
        if(unnormalise(&pair->depthmap_norm[LEFT],
                       &pair->projection[LEFT],
                       &pair->depthmap[LEFT]))
            return -1;

//       image_display(&pair->depthmap[LEFT]);

        if(unnormalise(&pair->depthmap_norm[RIGHT],
                       &pair->projection[RIGHT],
                       &pair->depthmap[RIGHT]))
            return -1;

//       image_display(&pair->depthmap[RIGHT]);
    }

    return 0;
}

static int undo_stereo(pair_t *pair)
{
    image_free(&pair->depthmap_norm[LEFT]);
    image_free(&pair->depthmap_norm[RIGHT]);
    image_free(&pair->depthmap[LEFT]);
    image_free(&pair->depthmap[RIGHT]);

    image_free(&pair->image_norm[LEFT]);
    image_free(&pair->image_norm[RIGHT]);
    image_free(&pair->image[LEFT]);
    image_free(&pair->image[RIGHT]);

    return 0;
}

